<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use \Doctrine\ORM\EntityManager;
use Teknisa\MAN\Services\ProdutividadeVendedor;


class ProdutividadeVendedorContinuo extends ProdutividadeVendedor{

	protected $entityManager;

	public function __construct(EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}

    private function setReportParameters(&$params, &$conditions) {
        $params = array(
            ':DATAINI'       => isset($conditions['DATAINI'])       ? $conditions['DATAINI']       : null,
            ':DATAFIN'       => isset($conditions['DATAFIN'])       ? $conditions['DATAFIN']       : null,
            ':CDFILIAL'      => isset($conditions['CDFILIAL'])      ? $conditions['CDFILIAL']      : null,
            ':TCDFILIAL'     => isset($conditions['TCDFILIAL'])     ? $conditions['TCDFILIAL']     : null,
            ':CDLOJA'        => isset($conditions['CDLOJA'])        ? $conditions['CDLOJA']        : null,
            ':TCDLOJA'       => isset($conditions['TCDLOJA'])       ? $conditions['TCDLOJA']       : null,
            ':CDVENDEDOR'    => isset($conditions['CDVENDEDOR'])    ? $conditions['CDVENDEDOR']    : null,
            ':TCDVENDEDOR'   => isset($conditions['TCDVENDEDOR'])   ? $conditions['TCDVENDEDOR']   : null,
            ':DTMOVTURCAIX'  => isset($conditions['DTMOVTURCAIX'])  ? $conditions['DTMOVTURCAIX']  : null,
            ':TDTMOVTURCAIX' => isset($conditions['TDTMOVTURCAIX']) ? $conditions['TDTMOVTURCAIX'] : null,
            ':CDPRODUTO'     => null,
            ':TCDPRODUTO'    => 'T' 
        );
    }

    public function getProdutividadeVendedor($conditions) {
    	$params = [];
        $this->setReportParameters($params, $conditions);
        $query = "
            SELECT TOTALFINAL.CDFILIAL, TOTALFINAL.NMFILIAL, TOTALFINAL.CDLOJA, TOTALFINAL.NMLOJA,
            		TOTALFINAL.DTABERTUR, TOTALFINAL.FILILOJA, TOTALFINAL.NRINSJURVEN AS RAZAO,
            		TOTALFINAL.CDVENDEDOR2 || ' - ' || TOTALFINAL.NMRAZSOCVEN AS NMRAZSOCVEN,		
            		SUM(TOTALFINAL.ACR) AS ACR, 
            		SUM(TOTALFINAL.DES) AS DES, 
            		SUM(TOTALFINAL.QTPRODVEND) AS QTDE,
            		ROUND(SUM(TOTALFINAL.TOTALVENDEDOR + TOTALFINAL.VRTXENTREGA2 + TOTALFINAL.VRREPITEM + NVL(TOTALFINAL.VRGORJETA, 0)), 2) AS VALOR_TOTAL, 
            		ROUND(SUM(TOTALFINAL.VRGORJETA), 2) AS VRGORJETA, 
            		ROUND(SUM(TOTALFINAL.TOTALVENDEDOR + TOTALFINAL.VRTXENTREGA2 + NVL(TOTALFINAL.VRGORJETA, 0)), 2) AS VRLIQUIDO,
            		ROUND(SUM(TOTALFINAL.VRREPITEM), 2) AS VRREPIQUE,
            		ROUND(SUM(TOTALFINAL.VRTXENTREGA), 2) AS VRTXENTREGA		
            	FROM (
            		SELECT FINAL.FILILOJA, FINAL.GARCON, FINAL.CDFILIAL, FINAL.NMFILIAL, FINAL.CDLOJA, FINAL.NMLOJA,
            				FINAL.CDVENDEDOR2, FINAL.NMRAZSOCVEN, FINAL.CDPRODNIVANTE, FINAL.NRINSJURVEN,
            				FINAL.ACR, FINAL.DES, FINAL.QTPRODVEND, FINAL.DTABERTUR,
            				NVL(FINAL.VRGORJETA, 0) AS VRGORJETA, (FINAL.VRTXENTREGA/FINAL.TOTQTPROD) AS VRTXENTREGA,
            				FINAL.TOTALVENDEDOR, (FINAL.VRTXENTREGA/FINAL.TOTQTPROD) AS VRTXENTREGA2, (FINAL.VRREPITEM/FINAL.TOTQTPROD) AS VRREPITEM
            			FROM (
            				SELECT TOTVENDEDOR.CDFILIAL || ' - ' || F.NMFILIAL || ' - ' || TOTVENDEDOR.CDLOJA || ' - ' || TOTVENDEDOR.NMLOJA AS FILILOJA,
            						NVL(TOTVENDEDOR.CDVENDEDOR, TOTVENDEDOR.VENDEDORVENDA) || ' - ' || NVL(TOTVENDEDOR.NMVENDEDORITEM, TOTVENDEDOR.NMVENDEDORVENDA) AS GARCON,
            						NVL(TOTVENDEDOR.CDVENDEDOR, TOTVENDEDOR.VENDEDORVENDA) AS CDVENDEDOR2, 
            						TOTVENDEDOR.NRINSJURVEN,
            						NVL(TOTVENDEDOR.NMVENDEDORITEM, TOTVENDEDOR.NMVENDEDORVENDA) AS NMRAZSOCVEN,
            						TOTVENDEDOR.CDFILIAL, F.NMFILIAL, TOTVENDEDOR.CDLOJA, TOTVENDEDOR.NMLOJA,
            						TOTVENDEDOR.CDPRODNIVANTE,
            						TRUNC(TOTVENDEDOR.VRUNITVEND, 2) AS VRUNIT, TOTVENDEDOR.VRACRITVEND AS ACR, 
            						TOTVENDEDOR.VRDESITVEND AS DES, TOTVENDEDOR.QTPRODVEND, 
            						NVL((CASE TOTVENDEDOR.IDCOBTXSERV 
            							WHEN 'S' 
            								THEN (NVL(TOTGORJ.TOTALGORJ, 0) * (NVL(TOTVENDEDOR.TOTALVENDEDOR, 1) / NVL(TOTCALCGORJ.TOTCALCGORJ, 1)))
            							WHEN 'N' 
            								THEN 0 
            						END), 0) AS VRGORJETA,
            						TOTVENDEDOR.NRSEQVENDA, NVL(TXENTREGA.VRTXENTREGA, 0) AS VRTXENTREGA, 
            						TOTVENDEDOR.VRREPIQUEVENDA AS VRREPITEM, NVL(TXENTREGA.VRTXENTREGA, 0) AS VRTXENTREGA2,
            						TOTVENDEDOR.TOTALVENDEDOR, TOTVENDEDOR.DTABERTUR,
            						COUNT(TOTVENDEDOR.NRSEQVENDA) OVER (PARTITION BY TOTVENDEDOR.CDFILIAL, TOTVENDEDOR.CDCAIXA, TOTVENDEDOR.NRSEQVENDA) AS TOTQTPROD
            					FROM (
            						SELECT I.CDFILIAL, I.CDCAIXA, L.CDLOJA, L.NMLOJA, I.NRSEQVENDA, I.CDVENDEDOR, VI.NMRAZSOCVEN AS NMVENDEDORITEM, 
            								V.CDVENDEDOR AS VENDEDORVENDA, VV.NMRAZSOCVEN AS NMVENDEDORVENDA,
            								P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND, P.CDPRODNIVANTE,
            								SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) AS TOTALVENDEDOR,
            								SUM(NVL(I.VRACRITVEND, 0)) AS VRACRITVEND, SUM(NVL(I.QTPRODVEND, 0)) AS QTPRODVEND, P.IDCOBTXSERV, SUM(NVL(I.VRDESITVEND, 0)) AS VRDESITVEND,
            								V.VRREPIQUEVENDA, V.DTABERTUR, NVL(VV.NRINSJURVEN, VI.NRINSJURVEN) AS NRINSJURVEN
            							FROM ITEMVENDA I
            								JOIN PRODUTO P
            									ON I.CDPRODUTO = P.CDPRODUTO
            								JOIN VENDA V
            									ON V.CDFILIAL = I.CDFILIAL
            										AND V.CDCAIXA = I.CDCAIXA
            										AND V.NRSEQVENDA = I.NRSEQVENDA
            								JOIN LOJA L
            									ON V.CDFILIAL = L.CDFILIAL
            										AND V.CDLOJA = L.CDLOJA
            								LEFT JOIN VENDEDOR VV
            									ON V.CDVENDEDOR = VV.CDVENDEDOR
            								LEFT JOIN VENDEDOR VI
            									ON I.CDVENDEDOR = VI.CDVENDEDOR
            							WHERE V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
            								AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
            								AND I.CDPRODUTO <> NVL(L.CDPRODTAXASERV, 0)
            								AND I.CDPRODUTO <> NVL(L.CDPRODTAXAENTR, 0)
            								AND NVL(V.CDVENDEDOR, I.CDVENDEDOR) IS NOT NULL
            								AND V.IDSITUVENDA = 'O'
            								AND ((INSTR(:CDLOJA, L.CDFILIAL||L.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
            								AND ((INSTR(:DTMOVTURCAIX, '_'||I.CDFILIAL||I.CDCAIXA||TO_CHAR(V.DTABERTUR, 'DD/MM/YYYY HH24:MI:SS')) > 0) OR (:TDTMOVTURCAIX = 'T'))
            							GROUP BY I.CDFILIAL, I.CDCAIXA, L.CDLOJA, L.NMLOJA, I.NRSEQVENDA, 
            									I.CDVENDEDOR, VI.NMRAZSOCVEN, V.CDVENDEDOR, VV.NMRAZSOCVEN, P.CDARVPROD, 
            									P.NMPRODUTO, I.VRUNITVEND, P.CDPRODNIVANTE, P.IDCOBTXSERV, V.VRREPIQUEVENDA, V.DTABERTUR, NVL(VV.NRINSJURVEN, VI.NRINSJURVEN)
            					) TOTVENDEDOR
            					LEFT JOIN (
            						SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA, V.CDVENDEDOR, 
            								SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) AS TOTALGORJ
            							FROM ITEMVENDA I
            								JOIN PRODUTO P
            									ON I.CDPRODUTO = P.CDPRODUTO
            								JOIN VENDA V
            									ON V.CDFILIAL = I.CDFILIAL
            										AND V.CDCAIXA = I.CDCAIXA
            										AND V.NRSEQVENDA = I.NRSEQVENDA
            								JOIN LOJA L
            									ON V.CDFILIAL = L.CDFILIAL
            										AND V.CDLOJA = L.CDLOJA
            							WHERE V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
            								AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
            								AND I.CDPRODUTO = L.CDPRODTAXASERV
            								AND V.IDSITUVENDA = 'O'
            							GROUP BY I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA, V.CDVENDEDOR
            					) TOTGORJ
            						ON TOTVENDEDOR.CDFILIAL = TOTGORJ.CDFILIAL
            							AND TOTVENDEDOR.NRSEQVENDA = TOTGORJ.NRSEQVENDA
            							AND TOTVENDEDOR.CDCAIXA = TOTGORJ.CDCAIXA
            					LEFT JOIN (
            						SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA,
            							SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0)) AS TOTCALCGORJ
            							FROM ITEMVENDA I 
            								JOIN VENDA V
            									ON V.CDFILIAL = I.CDFILIAL
            										AND V.CDCAIXA = I.CDCAIXA
            										AND V.NRSEQVENDA = I.NRSEQVENDA
            								JOIN PRODUTO P
            									ON I.CDPRODUTO = P.CDPRODUTO
            							WHERE V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
            								AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
            								AND P.IDCOBTXSERV = 'S'
            								AND V.IDSITUVENDA = 'O'
            							GROUP BY I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA
            					) TOTCALCGORJ
            						ON TOTVENDEDOR.CDFILIAL = TOTCALCGORJ.CDFILIAL
            							AND TOTVENDEDOR.NRSEQVENDA = TOTCALCGORJ.NRSEQVENDA
            							AND TOTVENDEDOR.CDCAIXA = TOTCALCGORJ.CDCAIXA
            					JOIN (
            						SELECT CDFILIAL, NMFILIAL
            							FROM FILIAL
            							WHERE ((INSTR(:CDFILIAL, CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
            					) F
            						ON TOTVENDEDOR.CDFILIAL = F.CDFILIAL
            					LEFT JOIN (
            						SELECT V.CDFILIAL, V.NRSEQVENDA, V.CDCAIXA, SUM(NVL(I.VRACRITVEND, 0)) AS VRTXENTREGA, V.DTABERTUR
            							FROM ITEMVENDA I, VENDA V, LOJA L
            							WHERE I.CDFILIAL = V.CDFILIAL
            								AND I.CDCAIXA    = V.CDCAIXA
            								AND I.NRSEQVENDA = V.NRSEQVENDA
            								AND V.CDLOJA     = L.CDLOJA
            								AND V.CDFILIAL   = L.CDFILIAL
            								AND I.CDPRODUTO IN (L.CDPRODTAXAENTR)
            								AND I.CDPRODUTO <> NVL(L.CDPRODTAXASERV, 0)
            								AND V.IDSITUVENDA = 'O'
            								AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
            								AND ((INSTR(:CDLOJA, L.CDFILIAL||L.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
            								AND V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
            							GROUP BY V.CDFILIAL, V.NRSEQVENDA, V.CDCAIXA, V.DTABERTUR
            					) TXENTREGA
            						ON TOTVENDEDOR.CDFILIAL = TXENTREGA.CDFILIAL
            							AND TOTVENDEDOR.NRSEQVENDA = TXENTREGA.NRSEQVENDA
            							AND TOTVENDEDOR.CDCAIXA = TXENTREGA.CDCAIXA
            			) FINAL
            			WHERE ((INSTR(:CDVENDEDOR, FINAL.CDVENDEDOR2) > 0) OR (:TCDVENDEDOR = 'T'))
            	) TOTALFINAL
            	GROUP BY TOTALFINAL.CDFILIAL, TOTALFINAL.NMFILIAL, TOTALFINAL.CDLOJA, TOTALFINAL.NMLOJA,
            			TOTALFINAL.CDVENDEDOR2, TOTALFINAL.NMRAZSOCVEN, TOTALFINAL.DTABERTUR,
            			TOTALFINAL.FILILOJA, TOTALFINAL.NRINSJURVEN
            ORDER BY TOTALFINAL.FILILOJA, TOTALFINAL.DTABERTUR, TOTALFINAL.CDVENDEDOR2, TOTALFINAL.NMRAZSOCVEN
        ";
        
        $query =  strval(trim($query));
        $qryPrincipal = $this->entityManager->getConnection()->fetchAll($query, $params);
        return $qryPrincipal;
    }
}